@@ -1,7 +1,7 @@
 """pymirror"""
 from .config import *
-from .pymirror import *
+from .pymirror import PyMirror
 from .cli import *
 
-__version__ = '0.2.0'
+__version__ = '0.2.7'
 __description__ = 'Upload files to multiple free hosting servers'
