@@ -1,6 +1,6 @@
 import argparse
 
-import pymirror
+from .pymirror import PyMirror
 
 
 class CustomHelpFormatter(argparse.HelpFormatter):
@@ -29,7 +29,7 @@ def main(**kwargs):
                         action='help',
                         default=argparse.SUPPRESS,
                         help='Show this help message and exit')
-    parser.add_argument('-i', '--input', help='Path to the input file/folder')
+    parser.add_argument('-i', '--input', help='Path to the input file/folder', required=True)
     parser.add_argument('-s',
                         '--style',
                         help='Output style (default: lines)',
@@ -37,7 +37,7 @@ def main(**kwargs):
                         default='lines')
     parser.add_argument(
         '-m',
-        '--mirroredto',
+        '--more-links',
         help='Use mirrored.to to generate more likes (default: False)',
         action='store_true',
         default=False)
@@ -62,13 +62,13 @@ def main(**kwargs):
     parser.add_argument(
         '-l',
         '--log',
-        help='Log the current uploadto a file (default: False)',
+        help='Show logs and save it to a file (default: False)',
         action='store_true',
         default=False)
 
     args = parser.parse_args()
 
-    mirror = pymirror.PyMirror().uploader(args)
+    mirror = PyMirror().uploader(args)
 
 
 if __name__ == '__main__':
