@@ -2,7 +2,9 @@
 # coding: utf-8
 
 import argparse
+import concurrent.futures
 from glob import glob
+import importlib.util
 import json
 import multiprocessing as mp
 import os
@@ -20,8 +22,8 @@ import tarfile
 import time
 import traceback
 from pathlib import Path
-from types import SimpleNamespace
 
+from dracula import DraculaPalette as dp
 from rich.console import Console
 from loguru import logger
 from selenium import webdriver
@@ -46,7 +48,7 @@ class KeyboardInterruptHandler:
         sys.tracebacklimit = 0
         print('', end='\r')
         time.sleep(0.5)
-        console.print('[#f1fa8c]Quitting...')
+        console.print(f'[{dp.y}]Quitting...')
         logger.info('Interrupted by the user.')
         sys.exit(0)
 
@@ -67,6 +69,149 @@ def fmt(prog):
     return CustomHelpFormatter(prog)
 
 
+class StartDrive:
+    def __init__(self):
+        pass
+
+    def start_driver(headless=True):
+        check_ublock = [
+            x for x in glob(f'{Config.PROJECT_PATH}/*')
+            if Path(x).name == 'ublock_latest.xpi'
+        ]
+        if not check_ublock:
+            PyMirror.download_ublock()
+
+        options = Options()
+        profile = webdriver.FirefoxProfile()
+        profile.set_preference('media.volume_scale', '0.0')
+        if headless:
+            options.headless = True
+        try:
+            driver = webdriver.Firefox(options=options,
+                                       firefox_profile=profile,
+                                       service_log_path=os.path.devnull)
+            driver.install_addon(Config.UBLOCK, temporary=True)
+            return driver
+        except WebDriverException:
+            os.environ['WDM_LOG_LEVEL'] = '0'
+            os.environ['WDM_PRINT_FIRST_LINE'] = 'False'
+            os.environ['WDM_LOCAL'] = '1'
+            try:
+                driver = webdriver.Firefox(
+                    executable_path=GeckoDriverManager().install(),
+                    options=options,
+                    firefox_profile=profile,
+                    service_log_path=os.path.devnull)
+            except ValueError:
+                raise Exception('Could not find Firefox!')
+            driver.install_addon(Config.UBLOCK, temporary=True)
+            return driver
+
+        except Exception as e:
+            SeleniumExceptionInfo(e)
+            console.print(f'[{dp.y}]Something is wrong with Selenium! '
+                          'Try again or remove the `--more-links` flag')
+
+
+class MoreLinks:
+    def __init__(self, file, headless=True):
+        self.file = file
+        self.driver = StartDrive.start_driver(headless)
+
+    def usaupload(self):
+        self.driver.get('https://usaupload.com/register_non_user')
+        time.sleep(2)
+        self.driver.find_element_by_id('add_files_btn').send_keys(self.file)
+        self.driver.find_element_by_class_name('upload-button').click()
+        while True:
+            time.sleep(1)
+            try:
+                link = self.driver.find_element_by_class_name(
+                    'col-xs-4').get_attribute('dtfullurl')
+                if link:
+                    break
+            except:
+                continue
+        return link
+
+    def filesharego(self):
+        self.driver.get('https://www.filesharego.com')
+        time.sleep(2)
+        for e in self.driver.find_elements_by_class_name('nav-item'):
+            if 'Upload File' in e.text:
+                e.click()
+                break
+        time.sleep(2)
+        self.driver.find_element_by_class_name('dz-hidden-input').send_keys(
+            self.file)
+        while True:
+            try:
+                time.sleep(1)
+                link_id = self.driver.find_element_by_id('copy').get_attribute(
+                    'data-id')
+                link = self.driver.find_element_by_id(link_id).get_attribute(
+                    'value')
+                if link:
+                    break
+            except:
+                continue
+        return link
+
+    def filepizza(self):
+        self.driver.get('https://file.pizza/')
+        self.driver.find_element_by_css_selector(
+            '.select-file-label > input:nth-child(1)').send_keys(self.file)
+        while True:
+            time.sleep(1)
+            link = self.driver.find_element_by_class_name('short-url').text
+            if link:
+                break
+        link = link.replace('or, for short: ', '')
+        return link
+
+    def expirebox(self):
+        self.driver.get('https://expirebox.com/')
+        time.sleep(2)
+        self.driver.find_element_by_id('fileupload').send_keys(self.file)
+        while True:
+            time.sleep(1)
+            try:
+                link = self.driver.find_element_by_css_selector(
+                    'div.input-group:nth-child(3) > input:nth-child(1)'
+                ).get_attribute('value')
+                if link:
+                    break
+            except:
+                continue
+        return link
+
+    def filepost(self):
+        self.driver.get('https://filepost.io/')
+        self.driver.find_element_by_css_selector(
+            '.drop-region > input:nth-child(4)').send_keys(self.file)
+        while True:
+            time.sleep(1)
+            try:
+                e = self.driver.find_element_by_css_selector(
+                    'div.buttons:nth-child(3) > a:nth-child(1)')
+                link = e.get_attribute('href').split('&body=')[1]
+                if link:
+                    break
+            except:
+                continue
+        return link
+
+    @staticmethod
+    def process(fun):
+        try:
+            link = fun()
+            all_links.append(link)
+            console.print(f'[[{dp.g}] OK [/{dp.g}]]', link)
+            return link
+        except Exception:
+            return
+
+
 class PyMirror:
     def __init__(self):
         pass
@@ -127,13 +272,13 @@ class PyMirror:
             response = os.system(f'ping -c 1 {ip[0]} > /dev/null 2>&1')
         if response in [0, 256, 512]:
             console.print(
-                f'[[#50fa7b] OK [/#50fa7b]] [#8be9fd]{ip[1]}[/#8be9fd] is online!'
+                f'[[{dp.g}] OK [/{dp.g}]] [{dp.c}]{ip[1]}[/{dp.c}] is online!'
             )
             logger.info(f'{ip[1]} is online!')
             return True
         else:
             console.print(
-                f'[[#ff5555] ERROR! [/#ff5555]] [#8be9fd]{ip[1]}[/#8be9fd] is down!'
+                f'[[{dp.r}] ERROR! [/{dp.r}]] [{dp.c}]{ip[1]}[/{dp.c}] is down!'
             )
             logger.warning(f'{ip[1]} is offline!')
             return False
@@ -171,7 +316,7 @@ class PyMirror:
             if args.number:
                 if n == args.number:
                     break
-            if res is False or k == 'mirroredto':
+            if res is False:
                 continue
             try:
                 signal.signal(signal.SIGALRM, lambda x, y: 1 / 0)
@@ -181,7 +326,7 @@ class PyMirror:
                 link = PyMirror.curl(data, k, rfile)
                 if 'bad gateway' in link.lower() or 'error' in link.lower():
                     raise Exception
-                console.print('[[#50fa7b] OK [/#50fa7b]]', link)
+                console.print(f'[[{dp.g}] OK [/{dp.g}]]', link)
                 all_links.append(link)
                 logger.info(f'[ OK ] {link}')
                 times.append(time.time() - start)
@@ -218,41 +363,145 @@ class PyMirror:
             f'curl -sLo {Config.PROJECT_PATH}/ublock_latest.xpi {latest}'
         ).read()
         console.print(
-            '\nYou\'re running the "--mirroredto" flag '
+            '\nYou\'re running the "--more-links" flag '
             'for the first time. Please wait until everything is ready. '
             'This is a one-time thing.\n',
             style='#f1fa8c')
-        time.sleep(5)
 
-    def mirroredto(file: str):
-        def start_driver():
-            options = Options()
-            options.headless = True
-            try:
-                driver = webdriver.Firefox(options=options,
-                                           service_log_path=os.path.devnull)
-                driver.install_addon(Config.UBLOCK, temporary=True)
-                return driver
-            except WebDriverException:
-                os.environ['WDM_LOG_LEVEL'] = '0'
-                os.environ['WDM_PRINT_FIRST_LINE'] = 'False'
-                os.environ['WDM_LOCAL'] = '1'
-                driver = webdriver.Firefox(
-                    executable_path=GeckoDriverManager().install(),
-                    options=options,
-                    service_log_path=os.path.devnull)
-                driver.install_addon(Config.UBLOCK, temporary=True)
-                return driver
-            except Exception as e:
-                SeleniumExceptionInfo(e)
-                sys.exit(1)
+    def more_links(file: str):
+        def mirror_services(driver, batch):
+            def mirroredto(driver, batch):
+                try:
+                    time.sleep(2)
+                    driver.get('https://www.mirrored.to/')
+                    time.sleep(5)
+                    html = driver.find_element_by_tag_name('html')
+                    _ = [html.send_keys(Keys.ARROW_DOWN) for _ in range(3)]
 
-        check_ublock = [
-            x for x in glob(f'{Config.PROJECT_PATH}/*')
-            if Path(x).name == 'ublock_latest.xpi'
-        ]
-        if not check_ublock:
-            PyMirror.download_ublock()
+                    for x in batch:
+                        try:
+                            driver.find_element_by_id(x.lower()).click()
+                        except Exception as e:
+                            PyMirror.SeleniumExceptionInfo(e)
+                            continue
+
+                    time.sleep(2)
+                    send_file = lambda selector: driver.find_element_by_css_selector(
+                        selector).send_keys(file)
+                    driver.find_element_by_css_selector(
+                        '#uploadifive-html_file_upload > input[type=file]:nth-child(3)'
+                    ).send_keys(file)
+                    time.sleep(1)
+
+                    driver.find_element_by_id('upload_button').click()
+                    time.sleep(5)
+
+                    while True:
+                        if driver.current_url != 'https://www.mirrored.to/':
+                            time.sleep(1)
+                            break
+
+                    link = driver.find_element_by_class_name('mlink').text
+                    driver.get(link)
+                    time.sleep(2)
+                    driver.find_element_by_class_name('secondary').click()
+                    time.sleep(2)
+
+                    start = time.time()
+
+                    while True:
+                        time.sleep(5)
+                        status = [
+                            x.text for x in driver.find_elements_by_class_name(
+                                'id_Success')
+                        ]
+                        if len(status) >= 8 or time.time() - start > 60:
+                            break
+
+                    for x in driver.find_elements_by_class_name('get_btn'):
+                        try:
+                            x.click()
+                        except Exception:
+                            continue
+
+                    current_window = driver.current_window_handle
+                    for handle in driver.window_handles:
+                        driver.switch_to.window(handle)
+                        try:
+                            LINK = driver.find_element_by_class_name(
+                                'code_wrap').text
+                            all_links.append(LINK)
+                            console.print(f'[[{dp.g}] OK [/{dp.g}]] {LINK}')
+                            logger.info(f'[ OK ] {LINK}')
+                            if handle != current_window:
+                                driver.close()
+                        except Exception:
+                            pass
+
+                except Exception as e:
+                    PyMirror.SeleniumExceptionInfo(e)
+
+                finally:
+                    driver.quit()
+
+            def multiup(driver, batch):
+                def cURL_request(url):
+                    cURL = shlex.split(url)
+                    out = subprocess.run(cURL, stdout=subprocess.PIPE)
+                    res = out.stdout.decode('UTF-8').replace('\\', '')
+                    try:
+                        res = json.loads(res)
+                    except JSONDecodeError:
+                        res = None
+                    return res
+
+                server = cURL_request(
+                    'curl -s https://www.multiup.org/api/get-fastest-server'
+                )['server']
+                selected_hosts_lst = [
+                    'filerio.in', 'drop.download', 'download.gg', 'uppit.com',
+                    'uploadbox.io'
+                ]
+                selected_hosts = ' '.join(
+                    [f'-F {x}=true' for x in selected_hosts_lst])
+                upload = cURL_request(
+                    f'curl -sF files[]=@{file} {selected_hosts} {server}')
+                link = upload['files'][0]['url'].replace(
+                    'download', 'en/mirror')
+                driver.get(link)
+
+                i = 0
+                j = 0
+
+                while True:
+                    time.sleep(1)
+                    if i != 0:
+                        if es_len != len(elements):
+                            j = 0
+                    driver.refresh()
+                    elements = driver.find_elements_by_class_name('host')
+                    es_len = len(elements)
+                    if es_len == 2:
+                        i += 1
+                    else:
+                        j += 1
+                    if j > i + 10:
+                        break
+                    elif len(elements) == len(selected_hosts_lst) + 1:
+                        break
+
+                for e in elements:
+                    if '(0)' in e.text:
+                        link = e.get_attribute('link')
+                        all_links.append(link)
+                        console.print(f'[[{dp.g}] OK [/{dp.g}]]', link)
+
+                driver.quit()
+
+            if batch:
+                mirroredto(driver, batch)
+            else:
+                multiup(driver, batch)
 
         first_batch = [
             'GoFileIo', 'TusFiles', 'OneFichier', 'ZippyShare', 'UsersDrive',
@@ -264,85 +513,20 @@ class PyMirror:
             'MegaupNet', 'dlupload', 'file-upload'
         ]
 
-        driver = start_driver()
+        start_driver = StartDrive.start_driver
+        drivers = [start_driver(), start_driver(), start_driver()]
+        if False in drivers:
+            return
+        batches = [first_batch, second_batch, None]
 
-        for ls in [first_batch, second_batch]:
-            try:
-                time.sleep(2)
-                driver.get('https://www.mirrored.to/')
-                time.sleep(5)
-                html = driver.find_element_by_tag_name('html')
-                _ = [html.send_keys(Keys.ARROW_DOWN) for _ in range(3)]
-
-                for x in ls:
-                    try:
-                        driver.find_element_by_id(x.lower()).click()
-                    except Exception as e:
-                        PyMirror.SeleniumExceptionInfo(e)
-                        continue
-
-                time.sleep(2)
-                send_file = lambda selector: driver.find_element_by_css_selector(
-                    selector).send_keys(file)
-                driver.find_element_by_css_selector(
-                    '#uploadifive-html_file_upload > input[type=file]:nth-child(3)'
-                ).send_keys(file)
-                time.sleep(1)
-
-                driver.find_element_by_id('upload_button').click()
-                time.sleep(5)
-
-                while True:
-                    if driver.current_url != 'https://www.mirrored.to/':
-                        time.sleep(1)
-                        break
-
-                link = driver.find_element_by_class_name('mlink').text
-                driver.get(link)
-                time.sleep(2)
-                driver.find_element_by_class_name('secondary').click()
-                time.sleep(2)
-
-                start = time.time()
-
-                while True:
-                    time.sleep(5)
-                    status = [
-                        x.text for x in driver.find_elements_by_class_name(
-                            'id_Success')
-                    ]
-                    if len(status) >= 8 or time.time() - start > 60:
-                        break
-
-                for x in driver.find_elements_by_class_name('get_btn'):
-                    try:
-                        x.click()
-                    except:
-                        continue
-
-                current_window = driver.current_window_handle
-                for handle in driver.window_handles:
-                    driver.switch_to.window(handle)
-                    try:
-                        LINK = driver.find_element_by_class_name(
-                            'code_wrap').text
-                        all_links.append(LINK)
-                        console.print(f'[[#50fa7b] OK [/#50fa7b]] {LINK}')
-                        logger.info(f'[ OK ] {LINK}')
-                        if handle != current_window:
-                            driver.close()
-                    except:
-                        continue
-
-                time.sleep(2)
-                driver.switch_to.window(current_window)
-
-            except Exception as e:
-                PyMirror.SeleniumExceptionInfo(e)
-                driver.quit()
-                break
-
-        driver.quit()
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            futures = []
+            results = [
+                executor.submit(mirror_services, driver, batch)
+                for driver, batch in zip(drivers, batches)
+            ]
+            for future in concurrent.futures.as_completed(results):
+                futures.append(future.result())
 
     def style_output(args, LINKs: dict):
         names = list(LINKs.keys())
@@ -372,9 +556,6 @@ class PyMirror:
             logger.add(Config.LOG_FILE, level='DEBUG')
             logger.add(sys.stderr, level='ERROR')
 
-        if args.mirroredto is True:
-            data['mirroredto'] = {'server': 'https://mirrored.to'}
-
         console.print('Press `CTRL+C` at any time to quit.', style='#f1fa8c')
 
         if args.check_status is True:
@@ -400,12 +581,30 @@ class PyMirror:
         links_raw = PyMirror.api_uploads(args, data, responses, rfile)
         # links = PyMirror.match_links(links_raw)
 
-        if args.mirroredto is True:
+        if args.more_links is True:
+            file_resolved = str(Path(rfile).resolve())
             try:
-                urls = PyMirror.mirroredto(str(Path(rfile).resolve()))
+                urls = PyMirror.more_links(file_resolved)
             except WebDriverException as e:
                 PyMirror.SeleniumExceptionInfo(e)
 
+            hosts = [
+                MoreLinks(file_resolved).filepizza,
+                MoreLinks(file_resolved).usaupload,
+                MoreLinks(file_resolved).filesharego,
+                MoreLinks(file_resolved).expirebox,
+                MoreLinks(file_resolved).filepost
+            ]
+
+            with concurrent.futures.ThreadPoolExecutor() as executor:
+                futures = []
+                results = [
+                    executor.submit(MoreLinks(file_resolved).process, fun)
+                    for fun in hosts
+                ]
+                for future in concurrent.futures.as_completed(results):
+                    futures.append(future.result())
+
         LINKs = {}
 
         for link in all_links:
@@ -426,7 +625,7 @@ class PyMirror:
                 os.remove(rfile)
         if Path(file).suffixes == ['.tar', '.gz']:
             os.remove(file)
-        console.rule('Results')
+        console.rule(f'Results: {len(all_links)}')
         print(output)
         os.rename(rfile, file)
         for x in all_links:
